
####################################################################################################################################
#################### Rgression, Probabilits, Intervalles de confiance et quelques tests statistiques #############################
####################################################################################################################################


####################################################################################################################################
############## Rgression linaire #################################################################################################
### on cherche  dtecter une corrlation entre deux variables quantitatives mesures sur les mmes individus

genomes=read.table("bacterial_genomes.txt", sep="\t", header=TRUE)	# ouverture du fichier "bacterial_genomes.txt"
attach(genomes);names(genomes)

plot(Genome_size,ORF_number,pch=16)			# plot(x,y) = reprsentation du nuage de points
cov(Genome_size,ORF_number)				# covariance
cor(Genome_size,ORF_number)				# coefficient de corrlation r de Pearson
res=lm(ORF_number ~ Genome_size)			# lm(y~x) = fonction de rgression linaire; on stocke les rsultats dans l'objet "res"
res								# intercept (b) et pente (a) de y=ax+b
summary(res)						# tests de significativit des coefficents
R2=cor(Genome_size,ORF_number)^2;R2			# R2 = carr du coeff de corrlation de Pearson (bien si > 70%)

plot(Genome_size,ORF_number,pch=16)			# reprsentation du nuage de points
abline(res,col="red",lwd=2)				# + AJOUT de la droite de regression


####################################################################################################################################
############## Lois de probabilits: calculs de probabilit ########################################################################

#Les distributions courantes sont programmes: Binomiale,Poisson,Normale,Uniforme,chi2...
#Plusieurs fonctions pour chaque distribution.
#Par exemple, pour la loi normale:
#dnorm() : fonction densit (density)
#pnorm() : fonction de rpartition (probability)
#qnorm() : fonction quantile (quantile)
#rnorm() : gnrateur alatoire (random)


### Exemple de calculs de probabilit avec une N(0,1)
# ici 0 est la moyenne et 1 est l'cart-type!
dnorm(c(-0.5,0,0.5),0,1)	# Prob[X=-0.5]	Prob[X=0]	Prob [X=0.5]
pnorm(c(-0.5,0,0.5),0,1)	# Prob[X<-0.5]	Prob[X<0]	Prob [X<0.5]
qnorm(c(0.25,0.5,0.75),0,1)	# quantiles  25,50(= mdiane) et 75 % 
rnorm(100,0,1)			# = 100 valeurs tires dans une N(0,1)
hist(rnorm(1000,0,1))

### Exemple de calculs de probabilit avec une loi Binomiale
# avec probabilit de germination = 0.8; sur n=100 graines je veux la probabilit que k=80 graines germent (k=nombre de succs)
# B(n,p)=B(100,0.8)
dbinom(80,100,prob=0.8)				# proba exacte
pbinom(80,100,prob=0.8)				# proba de "au maximum" 80 graines germes
pbinom(80,100,prob=0.8, lower.tail=F)	# proba de "plus de" 80 graines germes


####################################################################################################################################
############## Intervalles de confiance (IC) d'un paramtre,  partir de l'chantillon #############################################

### IC (95%) de la moyenne de la population  partir d'un chantillon
# Pour n>100 de prfrence: utilisation de la distribution normale
data=c(4,6,3,6,5,4,5,5,7)		# je n'ai pas mis 100 valeurs pour ne pas charger
m <- mean(data)				# moyenne calcule sur l'chantillon
s <- sd(data)				# cart-type calcul sur l'chantillon
n <- 200					# taille de l'chantillon
error <- qnorm(0.975)*s/sqrt(n)	# qnorm(0.975) = quantile 97,5% de la distribution normale;  s/sqrt(n) =erreur standard de la moyenne
left  <- m-error
right <- m+error
left;right # intervalle ayant 95% de chance de contenir la vraie moyenne  de la population

# Pour n<100 de prfrence: utilisation de la distribution de Student
m <- mean(data)				# moyenne calcule sur l'chantillon
s <- sd(data)				# cart-type calcul sur l'chantillon
n <- 20					# taille de l'chantillon
error <- qt(0.975,df=n-1)*s/sqrt(n)	# s/sqrt(n) =erreur standard de la moyenne
left  <- m-error
right <- m+error
left;right # intervalle ayant 95% de chance de contenir la vraie moyenne  de la population
# REMARQUE: si l'on veut un IC  99% ==> qnorm(0.995)*s/sqrt(n)


####################################################################################################################################
############## TESTS STATISTIQUES ##################################################################################################
# La plupart des tests statistiques courants (et bien dautres) sont programms dans R.
# - Tests de chi2, Test de Fisher pour la comparaison de variances 
# - Test de Normalit (Shapiro, ...), Test de Student pour la comparaison de moyennes.......
# Liste de tests sous R:      help.search("test",package="stats")

############ ATTENTION bien connatre l'hypothse nulle du test (H0) pour pouvoir correctement interprter !!! #####################

### TEST DE CHI2 DE CONFORMITE: tester la conformit d'une (ou plusieurs) valeur(s) par rapport  une (des) valeur(s) standard(s) ##
# H0 : conformit
# Exemple: proportion Mendelienne dans une descendance issue d'autofcondation d'une F1 htrozygote
freq_exp =c(1/4,1/2,1/4)	# proportions gnotypiques attendues (relatives) dans la descendance, pour un gne donn
freq_obs =c(22,53,25)		# proportions gnotypiques observes (absolues) parmis 100 individus descendants
chisq.test(freq_obs,p=freq_exp)


### TEST DE CHI2 DE  D'INDEPENDANCE: valuer la relation entre deux variables qualitatives  2 ou > 2 modalits ####################
# H0 : indpendance
# Exemple: Y-a-il un lien entre la toxicit d'chantillons de viande conditionne et la prsence d'une souche (P+) de la bactrie E. coli ?
# 2 variables qualitatives  2 modalits: "toxicit/non toxicit " de l'chantillon et "prsence/absence de la souche" dans l'chantillon 
#		P+		P-
#Tox		20		5
#Non_Tox	10		65
# on va faire un test de chi2 sur table de contingence des effectifs (nombre d'chantillons de viande dans chaque catgorie)

mat=matrix(c(20,5,10,65),nrow=2,byrow=T, dimnames=list(c("Tox", "N_Tox"), c("P+", "P-")))
mat
chisq.test(mat, correct =F)
obs=chisq.test(mat, correct =F)$observed
exp=chisq.test(mat, correct =F)$expected
obs-exp			# variables corrles: plus de Tox/P+ et de N_Tox/P- que sous l'hypothse d'indpendance


### TEST DE STUDENT de comparaison de moyennes de deux chantillons (1 variable quantitative mesure dans 2 chantillons A et B)###
# H0 : moyenne de A = moyenne de B
# la hauteur moyenne ( un temps de donn) de plantules d' A. thaliana de gnotype "sauvage" (Col_0)
# et d'un mutant pour le gne X1 (Mut_X1) est-elle la mme (hypothse H0)? 

## les donnes et les valeurs moyennes
Col_0=c(4.30,4.25,3.50,3.35,4.30,3.75,3.55,4.10,3.95,4.55,4.25,3.75,3.85,4.15,3.55,4.75,3.95,3.65)
Mut_X1=c(3.06,4.05,3.95,3.40,3.80,3.95,3.65,4,3.85,3.95,3.65,3.75,3.4)

mean(Col_0)		# moyenne de Col_0
mean(Mut_X1)	# moyenne de Mut_X1

## reprsentation graphique des donnes
boxplot(Col_0,Mut_X1,names=c("Col_0","Mut_X1"),col=c("white","darkgreen"),ylab="taille (cm)")
abline(h=mean(Col_0),col="black",lty=3,lwd=2)
abline(h=mean(Mut_X1),col="green",lty=3,lwd=2)
legend("topright", legend = c("Mut_X1","Col_A" ), text.col = c("darkgreen","black"))

## IMPORTANT: test pralables au test de student
# normalit des donnes de chaque chantillon (=H0)
shapiro.test(Col_0)	# tester la normalit de l'chantillon Col_0
shapiro.test(Mut_X1)	# tester la normalit de l'chantillon Mut_X1
# homognit des variances des 2 chantillons (=H0)
var.test(Col_0,Mut_X1) 

## test de Student de comparaison des moyennes de Col_0 et Mut_X1
t.test(Col_0,Mut_X1,var.equal = T)   				# test bilatral: H1 = il y a une diffrence de moyenne entre Col_0 et Mut_X1
t.test(Col_0,Mut_X1,var.equal = T, alternative = "greater")	# test unilatral (alternative = "greater"): H1= moyenne(Col_0) > moyenne(Mut_X1)
										# H1 = "two.sided" OU "less" OU "greater"

### REMARQUE SI LES VALEURS DANS CHAQUE ECHANTILLON NE SUIVENT PAS UNE LOI NORMALE ALORS
### Test non paramtrique de Mann & Whitney (Wilcoxon) #############################################################################
# mmes donnes (mme si elles suivent ici la loi normale)
wilcox.test(Col_0,Mut_X1)	
wilcox.test(Col_0,Mut_X1,alternative="greater")
# que remarque-t-on ici?
####################################################################################################################################




























###################################################################################################################################
################################## SUPPLEMENTS du TP2 #############################################################################
###################################################################################################################################

############## Lois de probabilits: calculs de probabilit et simulations #########################################################
### reprsentation graphique des distributions de probabilit d'une loi normale N(0,1))
par(mfrow=c(2,2))
x=seq(-3,3,0.001)				# liste de valeurs de la variable X pour lesquelles on veut calculer des probabilits
plot(x,dnorm(x,0,1),ylim=c(0,1))	# densit (loi) de probabilit
plot(x,pnorm(x,0,1),ylim=c(0,1))	# fonction de rpartition
y=seq(0,1,0.001)				# liste des quantiles (ici centiles)
plot(y,qnorm(y,0,1),ylim=c(-3,3))	# graphique des quantiles
plot(density(rnorm(1000,0,1)),xlim=c(-3,3),ylim=c(0,1),main="")	# histogramme (densit)  partir des valeurs tires d'une N(0,1)
####################################################################################################################################

############## Intervalles de confiance (IC) d'un paramtre,  partir de l'chantillon #############################################
### IC (95%) de la frquence d'un caractre dans la population,  partir de la proportion calcule sur un chantillon ##############
# Si n est grand (>100): utilisation de la distribution normale
p <- 0.7					# proportion calcule sur l'chantillon
n <- 100					# taille de l'chantillon  (faites la varier)
error <- qnorm(0.975)*sqrt((p*(1-p)/n))
left  <- p-error
right <- p+error
left;right # intervalle ayant 95% de chance de contenir la vraie proportion de la population
####################################################################################################################################


############## Test de chi2 d'adquation  une loi de distribution (ici  une Loi de Poisson) ######################################

# on cherche  modliser la probabilit de mutations multiples de certains gnes
# pour cela on essaie de voir si, d'aprs les bases de donnes de squences,
# la distribution du nombre de mutation par gne suit une loi de Poisson

# pour un gnome bactrien donns, et sur plusieurs clones
n_mutations= c(0,1,2,3,4,5,6)     #Nombre X de mutations par gne 
n_genes= c(90,80,35,12,5,1,0)     #Nombre de gnes prsentant X mutations 

#  occurence moyenne de mutation par gne = paramtre de la Loi de Poisson
lambda=weighted.mean(n_mutations,n_genes)    

# regroupement de classe d'au moins 5 gnes (pour X=0, X=1, X=2 et X > 3)
n_genes= c(90,80,35,18) 

# probabilits thoriques de Poisson
a=dpois(0, lambda); b=dpois(1, lambda); c=dpois(2, lambda)   # Prob [X = 0], Prob [X = 1], Prob [X = 2]
d=ppois(2, lambda, lower.tail=F)                              # Prob [X >  2]
a+b+c+d           

#d'o occurences thoriques sous la Loi de Poisson
p_theo=c(a,b,c,d)
# test de chi2 d'adquation
chisq.test(n_genes,p=p_theo)
# avec ddl = (n classes utilises pour le test)  n paramtres estims =  4-1=3
# Prob [X>0.8621] = 0.8346
# chisq.test(n_genes,p=p_theo)[[3]]
# si pas de regroupement en classes d'effectif minimum = 5, alors message "chi-squared approximation may be incorrect"
###################################################################################################################################

























